/////////////////////////////////////////////////////////////////////////////
//
// Undelete
//
// Copyright (C) 1998 Mark Russinovich and Bryce Cogswell
// Systems Internals - http://www.sysinternals.com
//
// See readme.txt for terms of use.
//
/////////////////////////////////////////////////////////////////////////////

//
// Filter.cpp : implementation file
//

#include "stdafx.h"
#include "Undelete.h"
#include "shlobj.h"
#include "ExtEdit.h"
#include "winioctl.h"
#include "..\dd\ioctlcmd.h"
#include "BinHeader.h"
#include "Filter.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFilter dialog


CFilter::CFilter()
	: CDialog(CFilter::IDD)
{
	//{{AFX_DATA_INIT(CFilter)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CFilter::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFilter)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CFilter, CDialog)
	//{{AFX_MSG_MAP(CFilter)
	ON_WM_CANCELMODE()
	ON_BN_CLICKED(IDDIRADD, OnDiradd)
	ON_BN_CLICKED(IDDIRREMOVE, OnDirremove)
	ON_BN_CLICKED(IDADD, OnAdd)
	ON_BN_CLICKED(IDREMOVE, OnRemove)
	ON_BN_CLICKED(IDOKAY, OnOkay)
	ON_WM_VKEYTOITEM()
	ON_LBN_SELCHANGE(IDC_FILTERLIST, OnSelchange)
	ON_WM_DELETEITEM()
	ON_LBN_SELCHANGE(IDC_DIRFILTERLIST, OnDirSelchange)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFilter message handlers

BOOL CFilter::OnInitDialog() 
{
	CDialog::OnInitDialog();
	POSITION pos;
	CListBox *filterList;
	CString	 *filter;

	// fill in extension filters
	filterList = (CListBox *) this->GetDlgItem( IDC_FILTERLIST );
	m_ExtensionListbox = filterList;
	pos = m_ExtensionFilters->GetHeadPosition();
	while( pos != NULL &&
		((filter = &m_ExtensionFilters->GetNext( pos )) != NULL)) {

		filterList->AddString( *filter );	
	}

	// fill in directory filters
	filterList = (CListBox *) this->GetDlgItem( IDC_DIRFILTERLIST );
	m_DirectoryListbox = filterList;
	pos = m_DirectoryFilters->GetHeadPosition();
	while( pos != NULL &&
		((filter = &m_ExtensionFilters->GetNext( pos )) != NULL)) {

		filterList->AddString( *filter );	
	}
	
	GetDlgItem( IDREMOVE )->EnableWindow( FALSE );
	GetDlgItem( IDDIRREMOVE )->EnableWindow( FALSE );

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CFilter::OnCancelMode() 
{
	CDialog::OnCancelMode();
	
	// TODO: Add your message handler code here
	
}

void CFilter::OnDiradd() 
{
    LPMALLOC pMalloc;    
	int	count, index;

    if (::SHGetMalloc(&pMalloc) == NOERROR)    
	{        
		BROWSEINFO bi;
        char pszBuffer[MAX_PATH];       
		CString newDirectory;
		LPITEMIDLIST pidl;
		BOOL selecting= TRUE;

        // Get help on BROWSEINFO struct - it's got all the bit settings.
		do {

			bi.hwndOwner = GetSafeHwnd();        
			bi.pidlRoot = NULL;
			bi.pszDisplayName = pszBuffer;
			bi.lpszTitle = _T("Select a Directory to Filter");
			bi.ulFlags = BIF_RETURNFSANCESTORS | BIF_RETURNONLYFSDIRS;
			bi.lpfn = NULL;        
			bi.lParam = 0;

			// This next call issues the dialog box.
			if ((pidl = ::SHBrowseForFolder(&bi)) != NULL) {
				if (::SHGetPathFromIDList(pidl, pszBuffer)) {
					
					// At this point pszBuffer contains the selected path 
					// make sure that its on a protected drive
					if( pszBuffer[1] != ':' ||
						!m_ValidDriveSize[pszBuffer[0]-'A'] ) {

						MessageBox( _T("The selected directory is not on a protected drive."),
							_T("Invalid Selection"), MB_OK|MB_ICONWARNING );

					} else {

						CString dirString( pszBuffer );
						CString filterString;
						int length;
						BOOL alreadyDefined = FALSE;

						count = m_DirectoryListbox->GetCount();
						length = dirString.GetLength();
						for( index = 0; index < count; index++ ) {

							m_DirectoryListbox->GetText( index, filterString );
							if( filterString.Left( min(length, filterString.GetLength()) ) == 
								dirString.Left(min(length, filterString.GetLength())) ) {

								alreadyDefined = TRUE;
							}
						}

						if( !alreadyDefined ) {

							m_DirectoryListbox->AddString( pszBuffer );
						
						} else {

							MessageBox( _T("The selected directory is already being filtered, or will subsume a\n"
								"filter defined for a directory beneath the selection."),
								_T("Duplicate Entry"), MB_OK|MB_ICONWARNING );
						}
					}
				}
				// Free the PIDL allocated by SHBrowseForFolder.
				pMalloc->Free(pidl);        

			} else {

				selecting = FALSE;
			}
			// Release the shell's allocator.        
			pMalloc->Release();    

		} while(selecting);
	}
}

void CFilter::OnDirremove() 
{
	int	item;

	if( (item = m_DirectoryListbox->GetCurSel( )) != LB_ERR ) {
		
		m_DirectoryListbox->DeleteString( item );	
		if( m_DirectoryListbox->SetCurSel( item ) == LB_ERR) {

			m_DirectoryListbox->SetCurSel( item-1 );
		}

		if( !m_DirectoryListbox->GetCount()) {

			GetDlgItem( IDDIRREMOVE )->EnableWindow( FALSE );
		}
	}
}

void CFilter::OnAdd() 
{
	CExtEdit extEdit;

	while( extEdit.DoModal() == IDOK &&
		extEdit.m_Extension.GetLength() ) {

		// make sure no '.' in string
		if( extEdit.m_Extension.Find(_T('.')) != -1 ) {

			MessageBox( _T("Extenions cannot contain '.'"),
				_T("Extension Error"), MB_OK|MB_ICONERROR );
			
		} else {

			// make sure string isn't already there
			if( m_ExtensionListbox->FindStringExact(-1, extEdit.m_Extension ) == 
				LB_ERR ) {

				m_ExtensionListbox->AddString( extEdit.m_Extension );
			
			} else {

				MessageBox( _T("The extension filter you entered is already defined."),
					_T("Duplicate Entry"), MB_OK|MB_ICONWARNING );
			}
		}
		extEdit.m_Extension = "";
	}
}

void CFilter::OnRemove() 
{
	int	item;

	if( (item = m_ExtensionListbox->GetCurSel( )) != LB_ERR ) {
		
		m_ExtensionListbox->DeleteString( item );
		if( m_ExtensionListbox->SetCurSel( item ) == LB_ERR) {

			m_ExtensionListbox->SetCurSel( item-1 );
		}

		if( !m_ExtensionListbox->GetCount()) {

			GetDlgItem( IDREMOVE )->EnableWindow( FALSE );
		}
	}
}

void CFilter::OnOkay() 
{
	CString filterString;
	int index, count;
	DWORD len, numFilters, sizeDirFilters;
	HANDLE driverHandle;
	PFILTERENTRY filters;
	PCHAR	dirFilters;
	POSITION pos;

	// clear existing filters
	m_DirectoryFilters->RemoveAll();
	m_ExtensionFilters->RemoveAll();

	count = m_DirectoryListbox->GetCount();
	for( index = 0; index < count; index++ ) {

		m_DirectoryListbox->GetText( index, filterString );
		m_DirectoryFilters->AddTail( filterString );
	}

	count = m_ExtensionListbox->GetCount();
	for( index = 0; index < count; index++ ) {

		m_ExtensionListbox->GetText( index, filterString );
		m_ExtensionFilters->AddTail( filterString );
	}

	// create the arrays
	sizeDirFilters = 0;
	for( pos = m_DirectoryFilters->GetHeadPosition(); pos != NULL; m_DirectoryFilters->GetNext(pos)) {

		filterString = m_DirectoryFilters->GetAt(pos);
		sizeDirFilters += filterString.GetLength()+1;
	}
	dirFilters = (PCHAR) malloc( sizeDirFilters );
	index = 0;
	for( pos = m_DirectoryFilters->GetHeadPosition(); pos != NULL; m_DirectoryFilters->GetNext(pos)) {

		filterString = m_DirectoryFilters->GetAt(pos);
		memcpy( &dirFilters[index], (LPCSTR) filterString, filterString.GetLength()+1);
		index += filterString.GetLength()+1;
	}

	numFilters = m_ExtensionFilters->GetCount();
	filters = (PFILTERENTRY) malloc( numFilters * sizeof(FILTERENTRY));
	index = 0;
	for( pos = m_ExtensionFilters->GetHeadPosition(); pos != NULL; m_ExtensionFilters->GetNext(pos)) {

		filterString = '.' + m_ExtensionFilters->GetAt(pos);
		strcpy( filters[index++].filter, (LPCSTR) filterString );
	}

	// Have to tell the driver the new filter
	if( (driverHandle = CreateFile( "\\\\.\\UNDELETE",
						  GENERIC_READ | GENERIC_WRITE,
						  0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,
						  NULL )) != ((HANDLE)-1) ) {

		// Have driver update the filter
		if ( ! DeviceIoControl(	driverHandle, UNDELETE_FILTER_UPDATE,
								filters, numFilters * sizeof(FILTERENTRY), 
								NULL, 0, &len, NULL ) ) {
			MessageBox( "Could not update Driver with extension filters", 
				"Undelete Error", MB_ICONERROR );
		}

		// Have driver update the filter
		if ( ! DeviceIoControl(	driverHandle, UNDELETE_DIR_FILTER_UPDATE,
								dirFilters, sizeDirFilters, 
								NULL, 0, &len, NULL ) ) {
			MessageBox( "Could not update Driver with directory filters", 
				"Undelete Error", MB_ICONERROR );
		}

		CloseHandle( driverHandle );

	} else {

		MessageBox( "Undelete driver is not loaded.\nUpdating Registry only.", 
				"Driver Update Warning", MB_ICONWARNING );
	}
	free( dirFilters );
	free( filters );

	EndDialog(IDOK);	
}


int CFilter::OnVKeyToItem(UINT nKey, CListBox* pListBox, UINT nIndex) 
{
	if( nKey == VK_DELETE ) {

		if( pListBox == m_ExtensionListbox ) {

			OnRemove();

		} else {

			OnDirremove();
		}
	} 
	return -1;
}

void CFilter::OnSelchange() 
{
	if( m_ExtensionListbox->GetCurSel( ) != LB_ERR ) {

		GetDlgItem( IDREMOVE )->EnableWindow( TRUE );

	} else {

		GetDlgItem( IDREMOVE )->EnableWindow( FALSE );
	}	
}

void CFilter::OnDirSelchange() 
{
	if( m_DirectoryListbox->GetCurSel( ) != LB_ERR ) {

		GetDlgItem( IDDIRREMOVE )->EnableWindow( TRUE );

	} else {

		GetDlgItem( IDDIRREMOVE )->EnableWindow( FALSE );
	}		
}
